@@ -1,18 +1,31 @@
 /*
- 				fitshead.c
-
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+*				fitshead.c
 *
-*	Part of:	The LDAC Tools
+* General functions for handling FITS file headers
 *
-*	Author:		E.BERTIN, DeNIS/LDAC
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 *
-*	Contents:	general functions for handling FITS file headers.
+*	This file part of:	AstrOmatic FITS/LDAC library
 *
-*	Last modify:	20/07/2010
+*	Copyright:		(C) 1995-2019 IAP/CNRS/SorbonneU
 *
-*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-*/
+*	License:		GNU General Public License
+*
+*	AstrOmatic software is free software: you can redistribute it and/or
+*	modify it under the terms of the GNU General Public License as
+*	published by the Free Software Foundation, either version 3 of the
+*	License, or (at your option) any later version.
+*	AstrOmatic software is distributed in the hope that it will be useful,
+*	but WITHOUT ANY WARRANTY; without even the implied warranty of
+*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*	GNU General Public License for more details.
+*	You should have received a copy of the GNU General Public License
+*	along with AstrOmatic software.
+*	If not, see <http://www.gnu.org/licenses/>.
+*
+*	Last modified:		04/12/2019
+*
+*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #ifdef	HAVE_CONFIG_H
 #include "config.h"
@@ -88,29 +101,38 @@ INPUT	pointer to catstruct.
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	25/09/2004
+VERSION	04/12/2019
  ***/
 void	readbasic_head(tabstruct *tab)
 
   {
    char		str[88];
-   char		key[12], name[16],
+   char		key[12], name[16], bitpix_key[16], naxis_key[16],
 		*filename;
    int		i;
    KINGSIZE_T	tabsize;
 
   filename = (tab->cat? tab->cat->filename : strcpy(name, "internal header"));
+  strncpy(bitpix_key, "BITPIX  ", 9);
+  strncpy(naxis_key, "NAXIS   ", 9);
 
-  if (fitsread(tab->headbuf, "BITPIX  ", &tab->bitpix, H_INT, T_LONG)
+#ifdef HAVE_CFITSIO
+  tab->isTileCompressed = (fitsread(tab->headbuf, "ZIMAGE  ",
+			str, H_STRING, T_STRING) == RETURN_OK)? 1 : 0;
+  if (tab->isTileCompressed) {
+    strncpy(bitpix_key, "ZBITPIX ", 9);
+    strncpy(naxis_key, "ZNAXIS  ", 9);
+  }
+#endif
+  if (fitsread(tab->headbuf, bitpix_key, &tab->bitpix, H_INT, T_LONG)
+	==RETURN_ERROR)
+    error(EXIT_FAILURE, "*Error*: Corrupted FITS header in ", filename);
+
+  if (fitsread(tab->headbuf, naxis_key, &tab->naxis, H_INT, T_LONG)
 	==RETURN_ERROR)
     error(EXIT_FAILURE, "*Error*: Corrupted FITS header in ", filename);
 
   tab->bytepix = tab->bitpix>0?(tab->bitpix/8):(-tab->bitpix/8);
-
-  if (fitsread(tab->headbuf, "NAXIS   ", &tab->naxis, H_INT, T_LONG)
-	==RETURN_ERROR)
-    error(EXIT_FAILURE, "*Error*: Corrupted FITS header in ", filename);
-
   tabsize = 0;
   if (tab->naxis>0)
     {
@@ -120,6 +142,11 @@ void	readbasic_head(tabstruct *tab)
     tabsize = 1;
     for (i=0; i<tab->naxis && i<999; i++)
       {
+#ifdef HAVE_CFITSIO
+       if (tab->isTileCompressed)
+         sprintf(key,"ZNAXIS%-2d", i+1);
+       else
+#endif
       sprintf(key,"NAXIS%-3d", i+1);
       if (fitsread(tab->headbuf, key, &tab->naxisn[i], H_INT, T_LONG)
 		==RETURN_ERROR)
@@ -131,6 +158,13 @@ void	readbasic_head(tabstruct *tab)
 /*random groups parameters (optional)*/
   tab->pcount = 0;
   fitsread(tab->headbuf, "PCOUNT  ", &tab->pcount, H_INT, T_LONG);
+
+#ifdef HAVE_CFITSIO
+  // CFITSIO TODO HACK
+  if (tab->isTileCompressed)
+    tab->pcount = 0;
+#endif
+
   tab->gcount = 1;
   fitsread(tab->headbuf, "GCOUNT  ", &tab->gcount, H_INT, T_LONG);
 
@@ -156,7 +190,7 @@ void	readbasic_head(tabstruct *tab)
 	1 : 0;
 
 /* Custom basic FITS parameters */
-  tab->bitsgn = 1;
+  tab->bitsgn = (tab->bitpix==BP_BYTE) ? 0 : 1;
   fitsread(tab->headbuf, "BITSGN  ", &tab->bitsgn, H_INT, T_LONG);
 
   if (fitsread(tab->headbuf, "IMAGECOD", str, H_STRING, T_STRING)==RETURN_OK)
@@ -271,7 +305,12 @@ int	readbintabparam_head(tabstruct *tab)
         key->htype = H_STRING;
         break;
       default:
+#ifdef HAVE_CFITSIO
+        // CFITSIO TODO dodgy
+    	key->ttype = T_FLOAT;
+#else
         error(EXIT_FAILURE, "*Error*: Unknown TFORM in ", cat->filename);
+#endif
       }
 
 /*--handle the special case of multimensional arrays*/
@@ -466,8 +505,8 @@ PURPOSE	Update a FITS header to make it "primary" (not extension)
 INPUT	Table structure.
 OUTPUT	RETURN_OK if tab header was already primary, or RETURN_ERROR otherwise.
 NOTES	-.
-AUTHOR	E. Bertin (IAP & Leiden observatory) C. Marmo (IAP)
-VERSION	11/06/2007
+AUTHOR	E. Bertin (IAP) C. Marmo (IAP)
+VERSION	30/08/2011
  ***/
 int	prim_head(tabstruct *tab)
 
@@ -476,12 +515,13 @@ int	prim_head(tabstruct *tab)
     return RETURN_ERROR;
   if (!strncmp(tab->headbuf, "XTENSION",8))
       {
-      strncpy(tab->headbuf, "SIMPLE  =                    T  "
+      strncpy(tab->headbuf, "SIMPLE  =                    T "
 	"/ This is a FITS file                            ", 80);
-/* fitsverify 4.13 (CFITSIO V3.002) return an error
-   if PCOUNT and GCOUNT are in a primary header (23/05/2007)*/
       removekeywordfrom_head(tab, "PCOUNT");      
       removekeywordfrom_head(tab, "GCOUNT");      
+      removekeywordfrom_head(tab, "TFIELDS");      
+      removekeywordfrom_head(tab, "EXTNAME");      
+      *tab->extname = '\0';
       return RETURN_ERROR;
       }
 
@@ -657,8 +697,8 @@ PURPOSE	Return the size of a binary-table field from its ``TFORM''.
 INPUT	TFORM string (see the FITS documentation).
 OUTPUT	size in bytes, or RETURN_ERROR if the TFORM is unknown.
 NOTES	-.
-AUTHOR	E. Bertin (IAP & Leiden observatory)
-VERSION	28/10/2009
+AUTHOR	E. Bertin (IAP)
+VERSION	10/11/2010
  ***/
 int	tsizeof(char *str)
 
@@ -667,7 +707,8 @@ int	tsizeof(char *str)
    char	*str2;
 
   str2 = str;
-  if (!(n = strtol(str, &str2, 10)))
+  n = strtol(str, &str2, 10);
+  if (str2==str)
     n = 1;
 
   switch ((int)*str2)
@@ -690,8 +731,8 @@ PURPOSE	Give the ``t_type'' of a binary-table field from its ``TFORM''.
 INPUT	TFORM string (see the FITS documentation).
 OUTPUT	size in bytes, or RETURN_ERROR if the TFORM is unknown.
 NOTES	-.
-AUTHOR	E. Bertin (IAP & Leiden observatory)
-VERSION	28/10/2009
+AUTHOR	E. Bertin (IAP)
+VERSION	29/08/2012
  ***/
 t_type	ttypeof(char *str)
 
