@@ -7,7 +7,7 @@
 *
 *	This file part of:	AstrOmatic software
 *
-*	Copyright:		(C) 1993-2013 Emmanuel Bertin -- IAP/CNRS/UPMC
+*	Copyright:		(C) 1993-2019 IAP/CNRS/SorbonneU
 *
 *	License:		GNU General Public License
 *
@@ -23,7 +23,7 @@
 *	along with AstrOmatic software.
 *	If not, see <http://www.gnu.org/licenses/>.
 *
-*	Last modified:		15/11/2013
+*	Last modified:		20/12/2019
 *
 *%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
@@ -57,7 +57,7 @@ OUTPUT	pointer to a copy of the input structure.
 NOTES	Actually, only FITS parameters are copied. Lower-level structures
 	such as those created by the WCS or TNX libraries are generated.
 AUTHOR	E. Bertin (IAP)
-VERSION	31/08/2002
+VERSION	06/12/2019
  ***/
 wcsstruct	*copy_wcs(wcsstruct *wcsin)
 
@@ -84,8 +84,11 @@ wcsstruct	*copy_wcs(wcsstruct *wcsin)
 
 /* Initialize other WCS structures */
   init_wcs(wcs);
-/* Invert projection corrections */
-  invert_wcs(wcs);
+/* Copy possible invert projection corrections */
+  if (wcsin->inv_x)
+    wcs->inv_x = wcs->prj->inv_x = poly_copy(wcsin->inv_x);
+  if (wcsin->inv_y)
+    wcs->inv_y = wcs->prj->inv_y = poly_copy(wcsin->inv_y);
 /* Find the range of coordinates */
   range_wcs(wcs);  
 
@@ -328,7 +331,7 @@ INPUT	tab structure.
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	15/11/2013
+VERSION	04/05/2018
  ***/
 wcsstruct	*read_wcs(tabstruct *tab)
 
@@ -354,7 +357,7 @@ wcsstruct	*read_wcs(tabstruct *tab)
    double	drota;
    int		j, l, naxis;
    char		name[16],
-		*buf, *filename, *ptr;
+		*buf, *filename;
 
   buf = tab->headbuf;
   filename = (tab->cat? tab->cat->filename : strcpy(name, "internal header"));
@@ -397,6 +400,7 @@ wcsstruct	*read_wcs(tabstruct *tab)
     }
 
   if (fitsfind(buf, "CD?_????")!=RETURN_ERROR)
+    {
 /*-- If CD keywords exist, use them for the linear mapping terms... */
     for (l=0; l<naxis; l++)
       for (j=0; j<naxis; j++)
@@ -404,6 +408,7 @@ wcsstruct	*read_wcs(tabstruct *tab)
         sprintf(str, "CD%d_%d", l+1, j+1);
         FITSREADF(buf, str, wcs->cd[l*naxis+j], l==j?1.0:0.0)
         }
+    }
   else if (fitsfind(buf, "PC?_????")!=RETURN_ERROR)
 /*-- ...If PC keywords exist, use them for the linear mapping terms... */
     for (l=0; l<naxis; l++)
@@ -441,49 +446,39 @@ wcsstruct	*read_wcs(tabstruct *tab)
     {
      char	*pstr;
      double	date;
-     int	biss, dpar[3];
 
 /*-- Coordinate reference frame */
-/*-- Search for an observation date expressed in Julian days */
+
+/*-- Search for an observation start date expressed in Julian days */
     FITSREADF(buf, "MJD-OBS ", date, -1.0);
     if (date<0.0)
       FITSREADF(buf, "MJDSTART", date, -1.0);
-/*-- Precession date (defined from Ephemerides du Bureau des Longitudes) */
 /*-- in Julian years from 2000.0 */
     if (date>0.0)
       wcs->obsdate = 2000.0 - (MJD2000 - date)/365.25;
     else
       {
-/*---- Search for an observation date expressed in "civilian" format */
+/*---- Search for an observation start date expressed in "civilian" format */
       FITSREADS(buf, "DATE-OBS", str, "");
-      if (*str)
-        {
-/*------ Decode DATE-OBS format: DD/MM/YY or YYYY-MM-DD */
-        for (l=0; l<3 && (pstr = strtok_r(l?NULL:str,"/- ", &ptr)); l++)
-          dpar[l] = atoi(pstr);
-        if (l<3 || !dpar[0] || !dpar[1] || !dpar[2])
-          {
-/*-------- If DATE-OBS value corrupted or incomplete, assume 2000-1-1 */
-          warning("Invalid DATE-OBS value in header: ", str);
-          dpar[0] = 2000; dpar[1] = 1; dpar[2] = 1;
-          }
-        else if (strchr(str, '/') && dpar[0]<32 && dpar[2]<100)
-          {
-          j = dpar[0];
-          dpar[0] = dpar[2]+1900;
-          dpar[2] = j;
-          }
+      wcs->obsdate = 0.0;
+      if (*str && str_to_date_wcs(str, &wcs->obsdate) != RETURN_OK)
+        warning("Invalid DATE-OBS value in header: ", str);
+      }
 
-        biss = (dpar[0]%4)?0:1;
-/*------ Convert date to MJD */
-        date = -678956 + (365*dpar[0]+dpar[0]/4) - biss
-			+ ((dpar[1]>2?((int)((dpar[1]+1)*30.6)-63+biss)
-		:((dpar[1]-1)*(63+biss))/2) + dpar[2]);
-        wcs->obsdate = 2000.0 - (MJD2000 - date)/365.25;
-        }
-      else
-/*------ Well if really no date is found */
-        wcs->obsdate = 0.0;
+/*-- Search for an observation end date expressed in Julian days */
+    FITSREADF(buf, "MJD-END ", date, -1.0);
+    if (date<0.0)
+      FITSREADF(buf, "MJDSTOP ", date, -1.0);
+/*-- in Julian years from 2000.0 */
+    if (date>0.0)
+      wcs->obsend = 2000.0 - (MJD2000 - date)/365.25;
+    else
+      {
+      wcs->obsend = 0.0;
+/*---- Search for an observation start date expressed in "civilian" format */
+      FITSREADS(buf, "DATE-END", str, "");
+      if (*str && str_to_date_wcs(str, &wcs->obsend) != RETURN_OK)
+        warning("Invalid DATE-END value in header: ", str);
       }
 
     FITSREADF(buf, "EPOCH", wcs->epoch, 2000.0);
@@ -624,7 +619,7 @@ INPUT	tab structure,
 OUTPUT	-.
 NOTES	-.
 AUTHOR	E. Bertin (IAP)
-VERSION	27/11/2013
+VERSION	20/12/2019
  ***/
 void	write_wcs(tabstruct *tab, wcsstruct *wcs)
 
@@ -650,7 +645,13 @@ void	write_wcs(tabstruct *tab, wcsstruct *wcs)
     {
     mjd = (wcs->obsdate-2000.0)*365.25 + MJD2000;
     addkeywordto_head(tab, "MJD-OBS ", "Modified Julian date at start");
-    fitswrite(tab->headbuf, "MJD-OBS ", &mjd, H_EXPO,T_DOUBLE);
+    fitswrite(tab->headbuf, "MJD-OBS ", &mjd, H_FLOAT,T_DOUBLE);
+    }
+  if (wcs->obsend!=0.0)
+    {
+    mjd = (wcs->obsend-2000.0)*365.25 + MJD2000;
+    addkeywordto_head(tab, "MJD-END ", "Modified Julian date at end");
+    fitswrite(tab->headbuf, "MJD-END ", &mjd, H_FLOAT,T_DOUBLE);
     }
   addkeywordto_head(tab, "RADESYS ", "Astrometric system");
   switch(wcs->radecsys)
@@ -709,6 +710,43 @@ void	write_wcs(tabstruct *tab, wcsstruct *wcs)
   }
 
 
+/******* wipe_wcs ***********************************************************
+PROTO	void wipe_wcs(tabstruct *tab)
+PURPOSE	Remove all WCS (World Coordinate System) info in a FITS header.
+INPUT	tab structure.
+OUTPUT	-.
+NOTES	-.
+AUTHOR	E. Bertin (IAP)
+VERSION	27/11/2013
+ ***/
+void	wipe_wcs(tabstruct *tab)
+
+  {
+  removekeywordfrom_head(tab, "CRVAL???");
+  removekeywordfrom_head(tab, "CTYPE???");
+  removekeywordfrom_head(tab, "CUNIT???");
+  removekeywordfrom_head(tab, "CRPIX???");
+  removekeywordfrom_head(tab, "CRDER???");
+  removekeywordfrom_head(tab, "CSYER???");
+  removekeywordfrom_head(tab, "CDELT???");
+  removekeywordfrom_head(tab, "CROTA???");
+  removekeywordfrom_head(tab, "CD?_????");
+  removekeywordfrom_head(tab, "PROJP_??");
+  removekeywordfrom_head(tab, "PV?_????");
+  removekeywordfrom_head(tab, "PC?_????");
+  removekeywordfrom_head(tab, "PC0??0??");
+  removekeywordfrom_head(tab, "EQUINOX?");
+  removekeywordfrom_head(tab, "RADESYS?");
+  removekeywordfrom_head(tab, "RADECSYS");
+  removekeywordfrom_head(tab, "LONPOLE?");
+  removekeywordfrom_head(tab, "LONGPOLE");
+  removekeywordfrom_head(tab, "LATPOLE?");
+  removekeywordfrom_head(tab, "WAT?????");
+
+  return;
+  }
+
+
 /******* end_wcs **************************************************************
 PROTO	void end_wcs(wcsstruct *wcs)
 PURPOSE	Free WCS (World Coordinate System) infos.
@@ -781,19 +819,20 @@ INPUT	WCS structure.
 OUTPUT	-.
 NOTES	.
 AUTHOR	E. Bertin (IAP)
-VERSION	13/07/2012
+VERSION	06/12/2019
  ***/
 void	invert_wcs(wcsstruct *wcs)
 
   {
    polystruct		*poly;
-   double		pixin[NAXIS],raw[NAXIS],rawmin[NAXIS];
+   double		pixin[NAXIS],pixout[NAXIS], raw[NAXIS],rawmin[NAXIS];
    double		*outpos,*outpost, *lngpos,*lngpost,
 			*latpos,*latpost,
-			lngstep,latstep, rawsize, epsilon;
+			lngstep,latstep, rawsize, epsilon,
+			errlng, errlat, maxerrlng, maxerrlat;
    int			group[] = {1,1};
 				/* Don't ask, this is needed by poly_init()! */
-   int		i,j,lng,lat,deg, tnxflag, maxflag;
+   int		i,j,lng,lat,deg, tnxflag, maxflag, maxflagflag;
 
 /* Check first that inversion is not straightforward */
   lng = wcs->wcsprm->lng;
@@ -807,6 +846,14 @@ void	invert_wcs(wcsstruct *wcs)
   else
     return;
 
+/* Compute the error margin of the inversion in degrees */
+  for (i=0; i<wcs->naxis; i++)
+    raw[i] = (wcs->naxisn[i] + 1.0) / 2.0;
+  epsilon = WCS_INVACCURACY * wcs_scale(wcs, raw);
+  if (epsilon < TINY)
+    error(EXIT_FAILURE, "*Error*: incorrect pixel scale for ",
+		wcs->wcsprm->pcode);
+
 /* We define x as "longitude" and y as "latitude" projections */
 /* We assume that PCxx cross-terms with additional dimensions are small */
 /* Sample the whole image with a regular grid */
@@ -820,6 +867,8 @@ void	invert_wcs(wcsstruct *wcs)
   outpost = outpos;
   lngpost = lngpos;
   latpost = latpos;
+  maxflag = 0;
+  maxerrlng = maxerrlat = 0.0;
   for (j=WCS_NGRIDPOINTS; j--; raw[lat]+=latstep)
     {
     raw[lng] = rawmin[lng];
@@ -839,80 +888,76 @@ void	invert_wcs(wcsstruct *wcs)
         }
       else
         {
-        raw_to_pv(wcs->prj,pixin[lng],pixin[lat], outpost, outpost+1);
+        raw_to_pv(wcs->prj, pixin[lng],pixin[lat], outpost,outpost+1);
+        pv_to_raw(wcs->prj, *outpost,*(outpost+1), &pixout[lng],&pixout[lat]);
+        if ((errlng = fabs(pixout[lng] - pixin[lng])) > maxerrlng)
+          maxerrlng = errlng;
+        if ((errlat = fabs(pixout[lat] - pixin[lat])) > maxerrlat)
+          maxerrlat = errlat;
         outpost += 2;
         }
       }
     }
 
-/* Invert "longitude" */
-/* Compute the extent of the pixel in reduced projected coordinates */
-  linrev(rawmin, wcs->lin, pixin);
-  pixin[lng] += ARCSEC/DEG;
-  linfwd(pixin, wcs->lin, raw);
-  rawsize = sqrt((raw[lng]-rawmin[lng])*(raw[lng]-rawmin[lng])
-		+(raw[lat]-rawmin[lat])*(raw[lat]-rawmin[lat]))*DEG/ARCSEC;
-  if (!rawsize)
-    error(EXIT_FAILURE, "*Error*: incorrect linear conversion in ",
-		wcs->wcsprm->pcode);
-  epsilon = WCS_INVACCURACY/rawsize;
-/* Find the lowest degree polynom */
-  poly = NULL;  /* to avoid gcc -Wall warnings */
-  maxflag = 1;
-  for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++)
-    {
-    if (deg>1)
-      poly_end(poly);
-    poly = poly_init(group, 2, &deg, 1);
-    poly_fit(poly, outpos, lngpos, NULL, WCS_NGRIDPOINTS2, NULL);
-    maxflag = 0;
-    outpost = outpos;
-    lngpost = lngpos;
-    for (i=WCS_NGRIDPOINTS2; i--; outpost+=2)
-      if (fabs(poly_func(poly, outpost)-*(lngpost++))>epsilon)
-        {
-        maxflag = 1;
-        break;
-        }
-    }
-  if (maxflag)
-    warning("Significant inaccuracy likely to occur in projection","");
-/* Now link the created structure */
-  wcs->prj->inv_x = wcs->inv_x = poly;
+  maxflag |= (maxerrlng > WCS_INVACCURACY || maxerrlat > WCS_INVACCURACY);
 
-/* Invert "latitude" */
-/* Compute the extent of the pixel in reduced projected coordinates */
-  linrev(rawmin, wcs->lin, pixin);
-  pixin[lat] += ARCSEC/DEG;
-  linfwd(pixin, wcs->lin, raw);
-  rawsize = sqrt((raw[lng]-rawmin[lng])*(raw[lng]-rawmin[lng])
-		+(raw[lat]-rawmin[lat])*(raw[lat]-rawmin[lat]))*DEG/ARCSEC;
-  if (!rawsize)
-    error(EXIT_FAILURE, "*Error*: incorrect linear conversion in ",
-		wcs->wcsprm->pcode);
-  epsilon = WCS_INVACCURACY/rawsize;
-/* Find the lowest degree polynom */
-  maxflag = 1;
-  for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++)
-    {
-    if (deg>1)
-      poly_end(poly);
-    poly = poly_init(group, 2, &deg, 1);
-    poly_fit(poly, outpos, latpos, NULL, WCS_NGRIDPOINTS2, NULL);
-    maxflag = 0;
-    outpost = outpos;
-    latpost = latpos;
-    for (i=WCS_NGRIDPOINTS2; i--; outpost+=2)
-      if (fabs(poly_func(poly, outpost)-*(latpost++))>epsilon)
-        {
-        maxflag = 1;
-        break;
-        }
-    }
-  if (maxflag)
-    warning("Significant inaccuracy likely to occur in projection","");
-/* Now link the created structure */
-  wcs->prj->inv_y = wcs->inv_y = poly;
+/// Test if polynomial inversion is required
+  if (maxflag) {
+/*-- Invert "longitude" */
+/*-- Find the lowest degree polynom */
+    poly = NULL;  /* to avoid gcc -Wall warnings */
+    maxflag = 1;
+    for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++)
+      {
+      if (deg>1)
+        poly_end(poly);
+      poly = poly_init(group, 2, &deg, 1);
+      poly_fit(poly, outpos, lngpos, NULL, WCS_NGRIDPOINTS2, NULL,
+	1.0e-9/WCS_NGRIDPOINTS2);
+      maxflag = 0;
+      outpost = outpos;
+      lngpost = lngpos;
+      for (i=WCS_NGRIDPOINTS2; i--; outpost+=2)
+        if (fabs(poly_func(poly, outpost)-*(lngpost++))>epsilon)
+          {
+          maxflag = 1;
+          break;
+          }
+      }
+
+/*-- Now link the created structure */
+    wcs->prj->inv_x = wcs->inv_x = poly;
+
+    maxflagflag = maxflag;
+
+/*-- Invert "latitude" */
+/*-- Find the lowest degree polynom */
+    maxflag = 1;
+    for (deg=1; deg<=WCS_INVMAXDEG && maxflag; deg++)
+      {
+      if (deg>1)
+        poly_end(poly);
+      poly = poly_init(group, 2, &deg, 1);
+      poly_fit(poly, outpos, latpos, NULL, WCS_NGRIDPOINTS2, NULL,
+	1.0e-9/WCS_NGRIDPOINTS2);
+      maxflag = 0;
+      outpost = outpos;
+      latpost = latpos;
+      for (i=WCS_NGRIDPOINTS2; i--; outpost+=2)
+        if (fabs(poly_func(poly, outpost)-*(latpost++))>epsilon)
+          {
+          maxflag = 1;
+          break;
+          }
+      }
+/*-- Now link the created structure */
+    wcs->prj->inv_y = wcs->inv_y = poly;
+
+    maxflagflag |= maxflag;
+
+    if (maxflagflag)
+      warning("Significant inaccuracy likely to occur in projection","");
+  }
 
 /* Free memory */
   free(outpos);
@@ -2123,6 +2168,27 @@ double  fmod_m90_p90(double angle)
   }
 
 
+/********************************* fmod_0_pmod *******************************/
+/*
+Fold input angle in the [0,+mod[ domain.
+*/
+double  fmod_0_pmod(double angle, double mod)
+  {
+  return angle>0.0? fmod(angle,mod) : fmod(angle,mod)+mod;
+  }
+
+
+/******************************* fmod_mmod_pmod ******************************/
+/*
+Fold input angle in the [-mod,+mod[ domain.
+*/
+double  fmod_mmod_pmod(double angle, double mod)
+  {
+  return angle>0.0? fmod(angle+mod,2.0*mod)-mod : fmod(angle-mod,2.0*mod)+mod;
+  }
+
+
+
 /********************************* fcmp_0_p360 *******************************/
 /*
 Compare angles in the [0,+360[ domain: return 1 if anglep>anglem, 0 otherwise.
@@ -2134,4 +2200,50 @@ int  fcmp_0_p360(double anglep, double anglem)
   return (int)((dval>0.0 && dval<180.0) || dval<-180.0);
   }
 
+/****** str_to_date_wcs *****************************************************/
+/*
+PROTO	int str_to_date_wcs(char *str, double *year)
+PURPOSE	Convert WCS DATE-OBS string to Julian Date (in years).
+INPUT	Input DATE-OBS string,
+	Output Julian date in years.
+OUTPUT	RETURN_OK if the date was read correctly, RETURN_ERROR otherwise.
+NOTES	-.
+AUTHOR	E. Bertin (IAP)
+VERSION	20/12/2019
+ ***/
+int	str_to_date_wcs(char *str, double *year) {
+
+   double	dpar[6] = {0.0},
+		date, jdsec, tmp, biss;
+   char		str2[80], *pstr, *ptr;
+   int		l, sflag;
+
+  strncpy(str2, str, 72);
+
+/* Decode DATE-OBS format: DD/MM/YYThh:mm:ss[.sss] or YYYY-MM-DDThh:mm:ss[.sss] */
+  sflag = strchr(str2, '/') != NULL;
+  for (l=0; l<5 && (pstr = strtok_r(l ? NULL : str2, "/-T: ", &ptr)); l++)
+    dpar[l] = atof(pstr);
+
+  if (l<3 || dpar[0] == 0.0 || dpar[1] == 0.0 || dpar[2] == 0.0) {
+/*-- If DATE-OBS value corrupted or incomplete, assume 2000-1-1 */
+    *year = 2000.0;
+    return RETURN_ERROR;
+  } else if (sflag && dpar[0] < 32.0 && dpar[2] < 100.0) {
+    tmp = dpar[0];
+    dpar[0] = dpar[2] + 1900.0;
+    dpar[2] = tmp;
+  }
+
+   biss = (((int)dpar[0]) % 4) ? 0.0 : 1.0;
+
+/* Convert date to MJD */
+  jdsec = (dpar[5] + dpar[4] * 60.0 + dpar[3] * 3600.0) / 86400.0;
+  date = ((365.0 * dpar[0] + dpar[0] / 4.0) - 678956.0 - biss
+	+ ((dpar[1] > 2.0? (floor((dpar[1] + 1.0) * 30.6) - 63.0 + biss)
+	: ((dpar[1] - 1.0) * (63.0 + biss)) / 2.0) + dpar[2])) + jdsec;
+  *year = 2000.0 - (MJD2000 - date) / 365.25;
+
+  return RETURN_OK;
+}
 
